#!/usr/bin/make -f
# Makefile header (contains common code)
# Version 2016.01.01


## VERSION TIMESTAMP ##


override __MODULE_VERSION__=\"`date +"%Y.%m.%d"`\"


## CONDITIONS AND ARG PROCESSING ##


# Warning flags
ifdef WARN
	ifeq ($(WARN),1)
		override WARN=-Wshadow -Wpointer-arith -Wcast-align -Wwrite-strings -Wmissing-prototypes -Wmissing-declarations -Wredundant-decls -Wnested-externs -Winline -Wuninitialized -Wconversion -Wstrict-prototypes -Wbad-function-cast -Wdouble-promotion
	else ifeq ($(WARN),2)
		override WARN=-Wtraditional
	else
		override WARN=
	endif
else
	override WARN=
endif

# Optimization flags
ifdef XOPTMZ
	ifeq ($(XOPTMZ),1)
		override XOPTMZ=-fno-signed-zeros -fno-math-errno -fno-signaling-nans -ftree-vectorize -ffinite-math-only -mlong-double-128
	else ifeq ($(XOPTMZ),2)
		override XOPTMZ=-fno-signed-zeros -fno-rounding-math -fno-trapping-math -fno-math-errno -fno-signaling-nans -fcx-limited-range -ffinite-math-only
	else ifeq ($(XOPTMZ),3)
		override XOPTMZ=-ftree-vectorize
	else ifeq ($(XOPTMZ),4)
		override XOPTMZ=-funsafe-math-optimizations
	else ifeq ($(XOPTMZ),5)
		override XOPTMZ=-ffast-math
	else ifeq ($(XOPTMZ),6)
		override XOPTMZ=-funsafe-math-optimizations -ftree-vectorize
	else ifeq ($(XOPTMZ),7)
		override XOPTMZ=-ffast-math -funsafe-math-optimizations -ftree-vectorize
	else ifeq ($(XOPTMZ),8)
		override XOPTMZ=-minline-all-stringops -ftree-vectorize -fomit-frame-pointer
	else
		override XOPTMZ=
	endif
else
	override XOPTMZ=
endif

# C-Standard flags
ifdef STD
	# GNU
	ifeq ($(STD),gnu11)
		override STD=-std=gnu11
	else ifeq ($(STD),gnu99)
		override STD=-std=gnu99
	# C
	else ifeq ($(STD),c99)
		override STD=-std=c99
	else
		override STD=-std=c11
	endif
else
	override STD=-std=c11
endif

# Cross-compile flags
ifdef CROSS_COMPILE
	# x86 and x86-64
	ifeq ($(CROSS_COMPILE),x86)
		override BITS=32
		override CC=x86-linux-gnu-gcc -m32
		override STRIP=x86-linux-gnu-strip
		override LD=x86-linux-gnu-ld
		override AS=x86-linux-gnu-as
		override RANLIB=x86-linux-gnu-ranlib
		override AR=x86-linux-gnu-ar
	else ifeq ($(CROSS_COMPILE),x86-64)
		override BITS=64
		override CC=x86_64-linux-gnu-gcc -m64
		override STRIP=x86_64-linux-gnu-strip
		override LD=x86_64-linux-gnu-ld
		override AS=x86_64-linux-gnu-as
		override RANLIB=x86_64-linux-gnu-ranlib
		override AR=x86_64-linux-gnu-ar
	# ARM
	else ifeq ($(CROSS_COMPILE),armhf)
		override CC=arm-linux-gnueabihf-gcc
		override STRIP=arm-linux-gnueabihf-strip
		override LD=arm-linux-gnueabihf-ld
		override AS=arm-linux-gnueabihf-as
		override RANLIB=arm-linux-gnueabihf-ranlib
		override AR=arm-linux-gnueabihf-ar
	else ifeq ($(CROSS_COMPILE),arm)
		override CC=arm-linux-gnueabihf-gcc
		override STRIP=arm-linux-gnueabihf-strip
		override LD=arm-linux-gnueabihf-ld
		override AS=arm-linux-gnueabihf-as
		override RANLIB=arm-linux-gnueabihf-ranlib
		override AR=arm-linux-gnueabihf-ar
	else ifeq ($(CROSS_COMPILE),armel)
		override CC=arm-linux-gnueabi-gcc
		override STRIP=arm-linux-gnueabi-strip
		override LD=arm-linux-gnueabi-ld
		override AS=arm-linux-gnueabi-as
		override RANLIB=arm-linux-gnueabi-ranlib
		override AR=arm-linux-gnueabi-ar
	else ifeq ($(CROSS_COMPILE),armsf)
		override CC=arm-linux-gnueabi-gcc
		override STRIP=arm-linux-gnueabi-strip
		override LD=arm-linux-gnueabi-ld
		override AS=arm-linux-gnueabi-as
		override RANLIB=arm-linux-gnueabi-ranlib
		override AR=arm-linux-gnueabi-ar
	# Android (ARM)
	else ifeq ($(CROSS_COMPILE),android)
		override CC=arm-linux-androideabi-gcc
		override STRIP=arm-linux-androideabi-strip
		override LD=arm-linux-androideabi-ld
		override AS=arm-linux-androideabi-as
		override RANLIB=arm-linux-androideabi-ranlib
		override AR=arm-linux-androideabi-ar
	# General
	else
		override CC=gcc
		override STRIP=strip
		override LD=ld
		override AS=as
		override RANLIB=ranlib
		override AR=ar
	endif
endif

# Build commands
ifndef COMPILE
	override CC=gcc
	override STRIP=strip
	override LD=ld
	override AS=as
	override RANLIB=ranlib
	override AR=ar
else ifdef LLVM
	override CC=llvm
	override STRIP=strip
	override LD=llvm-link
	override AS=llvm-as
	override RANLIB=llvm-ranlib
	override AR=llvm-ar
endif

# Alternate linker
ifndef CROSS_COMPILE
	ifdef LINKER
		ifeq ($(LINKER),gold)
			override LD=gold
			override LINKER=-fuse-ld=gold
		else ifeq ($(LINKER),bfd)
			override LD=bfd
			override LINKER=-fuse-ld=bfd
		else
			override LD=ld
			override LINKER=
		endif
	endif
endif

# Architecture flags
ifdef ARCH
	# Intel
	ifeq ($(ARCH),intel)
		override ARCH=-mtune=intel
	else ifeq ($(ARCH),i486)
		override ARCH=-march=i486
	else ifeq ($(ARCH),i586)
		override ARCH=-march=i586
	else ifeq ($(ARCH),i686)
		override ARCH=-march=i686
	else ifeq ($(ARCH),pentium)
		override ARCH=-march=pentium
	else ifeq ($(ARCH),pentium2)
		override ARCH=-march=pentium2
	else ifeq ($(ARCH),pentium3)
		override ARCH=-march=pentium3
	else ifeq ($(ARCH),pentium4)
		override ARCH=-march=pentium4
	else ifeq ($(ARCH),sandybridge)
		override ARCH=-march=sandybridge
	else ifeq ($(ARCH),ivybridge)
		override ARCH=-march=ivybridge
	else ifeq ($(ARCH),haswell)
		override ARCH=-march=haswell
	else ifeq ($(ARCH),broadwell)
		override ARCH=-march=broadwell
	else ifeq ($(ARCH),bonnell)
		override ARCH=-march=bonnell
	else ifeq ($(ARCH),silvermont)
		override ARCH=-march=silvermont
	# AMD
	else ifeq ($(ARCH),k6)
		override ARCH=-march=k6
	else ifeq ($(ARCH),athlon)
		override ARCH=-march=athlon
	else ifeq ($(ARCH),opteron)
		override ARCH=-march=opteron
	# ARM
	else ifeq ($(ARCH),armv6)
		override ARCH=-march=armv6
	else ifeq ($(ARCH),armv7)
		override ARCH=-march=armv7
	else ifeq ($(ARCH),armv7-a)
		override ARCH=-march=armv7-a
	else ifeq ($(ARCH),armv7-r)
		override ARCH=-march=armv7-r
	else ifeq ($(ARCH),armv7-m)
		override ARCH=-march=armv7-m
	else ifeq ($(ARCH),armv8-a)
		override ARCH=-march=armv8-a
	else ifeq ($(ARCH),armv8-a+crc)
		override ARCH=-march=armv8-a+crc
	else ifeq ($(ARCH),iwmmxt)
		override ARCH=-march=iwmmxt
	# General
	else
		override ARCH=
	endif
else
	override ARCH=
endif

# 32 or 64 bits
ifdef BITS
	ifeq ($(BITS),64)
		override BITS=-m64
	else ifeq ($(BITS),32)
		override BITS=-m32
	else
		override BITS=
	endif
else
	override BITS=
endif

# ENDIANNESS
ifdef ENDIAN
	ifeq ($(ENDIAN),big)
		override ENDIAN=-mbig-endian
	else
		override ENDIAN=-mlittle-endian
	endif
else
	override ENDIAN=
endif

# Floating-point ABI
ifdef FLOATABI
	ifeq ($(FLOATABI),soft)
		override FLOATABI=-mfloat-abi=soft
	else ifeq ($(FLOATABI),softfp)
		override FLOATABI=-mfloat-abi=softfp
	else
		override FLOATABI=-mfloat-abi=hard
	endif
else
	override FLOATABI=
endif

override SHELL=/bin/sh

# Python version
ifdef PYVERSION
	ifeq ($(PYVERSION),3.4)
		override PYVERSION=3.4
		override CPYTHON=34
	else ifeq ($(PYVERSION),3.5)
		override PYVERSION=3.5
		override CPYTHON=35
	else ifeq ($(PYVERSION),3.6)
		override PYVERSION=3.6
		override CPYTHON=36
	else ifeq ($(PYVERSION),3.7)
		override PYVERSION=3.7
		override CPYTHON=37
	else ifeq ($(PYVERSION),3.8)
		override PYVERSION=3.8
		override CPYTHON=38
	else
		override PYVERSION=3.4
		override CPYTHON=34
	endif
else
	override PYVERSION=3.4
	override CPYTHON=34
endif


## PROFILES ##


ifdef INTEL
	override STD=-std=c11
	override CC=gcc
	override SHELL=/bin/sh
	override ARCH=-mtune=intel
	override XOPTMZ=
	override WARN=
	override PYVERSION=3.4
	override CPYTHON=34
	ifeq ($(INTEL),32)
		override BITS=-m32
	else
		override BITS=-m64
	endif
endif

ifdef dcj
	ifeq ($(dcj),1)
		override STD=-std=c11
		override ARCH=-march=haswell
		override BITS=-m64
		override CC=gcc
		override STRIP=strip
		override LD=ld
		override AS=as
		override RANLIB=ranlib
		override AR=ar
		override XOPTMZ=-fno-signed-zeros -fno-math-errno -fno-signaling-nans -ftree-vectorize -ffinite-math-only -mlong-double-128
		override PYVERSION=3.4
		override CPYTHON=34
		override SHELL=/bin/sh
		override WARN=-Wshadow -Wpointer-arith -Wcast-align -Wwrite-strings -Wmissing-prototypes -Wmissing-declarations -Wredundant-decls -Wnested-externs -Winline -Wuninitialized -Wconversion -Wstrict-prototypes -Wbad-function-cast
	else ifeq ($(dcj),rpi)
		override STD=-std=c11
		override ARCH=-march=armv6
		override BITS=-m32
		override ENDIAN=-mlittle-endian
		override FLOATABI=-mfloat-abi=hard
		override CROSS_COMPILE=armhf
		override CC=arm-linux-gnueabihf-gcc
		override STRIP=arm-linux-gnueabihf-strip
		override LD=arm-linux-gnueabihf-ld
		override AS=arm-linux-gnueabihf-as
		override RANLIB=arm-linux-gnueabihf-ranlib
		override AR=arm-linux-gnueabihf-ar
		override XOPTMZ=-fno-signed-zeros -fno-math-errno -fno-signaling-nans -ftree-vectorize -ffinite-math-only -mlong-double-128
		override PYVERSION=3.4
		override CPYTHON=34
		override SHELL=/bin/sh
		override WARN=-Wshadow -Wpointer-arith -Wcast-align -Wwrite-strings -Wmissing-prototypes -Wmissing-declarations -Wredundant-decls -Wnested-externs -Winline -Wuninitialized -Wconversion -Wstrict-prototypes -Wbad-function-cast -Wdouble-promotion
	endif
endif


## SPECIAL LLVM+CLANG FEATURES ##


ifdef LLVM_PIE
	override LLVM_PIE=-enable-pie
endif
# *.c/*.cpp => *.ll (LLVM IR/Intermediate-Language file)
override LLVM_INTERMEDIATE=clang $(ARCH) $(BITS) $(STD) -O3 -DNDEBUG -g0 -ggdb0 -funroll-loops -pthread -ffunction-sections -fdata-sections -S -emit-llvm
# *.c/*.cpp => *.bc (LLVM Bytecode file)
override LLVM_BYTECODE=clang $(ARCH) $(BITS) $(STD) -O3 -DNDEBUG -g0 -ggdb0 -funroll-loops -pthread -ffunction-sections -fdata-sections -flto -emit-llvm -c
# LLVM Bytecode (*.bc) => LLVM AT&T Assembly (*.s)
override LLVM_BC2ATTASM=llc $(ARCH) -O=3 -data-sections $(LLVM_PIE) -filetype=asm --x86-asm-syntax=att
# LLVM Bytecode (*.bc) => LLVM INTEL Assembly (*.s)
override LLVM_BC2INTELASM=llc $(ARCH) -O=3 -data-sections $(LLVM_PIE) -filetype=asm --x86-asm-syntax=intel
# LLVM Bytecode (*.bc) => Object file (*.o)
override LLVM_BC2OBJ=llc $(ARCH) -O=3 -data-sections $(LLVM_PIE) -filetype=obj
# *.c/*.cpp => *.ast (Clang AST file)
override CLANG_AST=clang $(ARCH) $(STD) -O3 -S -emit-ast
# Run Clang preprocessor on src and output preprocessed code (*.i)
override CLANG_PREPRO=clang $(ARCH) $(STD) -E
# Pre-compiled headers (*.pch)
override COMPILED_HEADERS=clang $(ARCH) $(STD) -x c-header


# To execute LLVM *.bc files, run
# lli ./FILE.bc ARGS
