/**
@author Devyn Collier Johnson <DevynCJohnson@Gmail.com>
@copyright LGPLv3

@brief Output the name of the current operating-system
@version 2016.01.01

@section LICENSE
GNU Lesser General Public License v3
Copyright (c) Devyn Collier Johnson, All rights reserved.

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 3.0 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library.
*/

// gcc -Wall -Wextra -pedantic -O3 -Wl,-O3 -g0 -ggdb0 -s -fwhole-program -funroll-loops -pthread -pipe -ffunction-sections -fdata-sections -std=c11 -o ./ostype ./ostype.c && strip --remove-section=.note --remove-section=.comment --strip-debug --strip-unneeded ./ostype


#include <stdio.h>


int main(void) {
    // IBM AIX
    #ifdef _AIX
    printf("AIX");
    #endif
    // OSX, iOS, NeXTSTEP, and Darwin
    #if defined(__APPLE__) && defined(__MACH__)
    #include <TargetConditionals.h>
    #if TARGET_IPHONE_SIMULATOR == 1
    printf("iOS");
    #elif TARGET_OS_IPHONE == 1
    printf("iOS");
    #elif TARGET_OS_MAC == 1
    printf("OSX");
    #endif
    #elif !defined(__APPLE__) && defined(__MACH__)
    printf("NeXTSTEP");
    #endif
    // BSD (DragonFly BSD, FreeBSD, OpenBSD, NetBSD)
    #if defined(__unix__) && defined(BSD)
    #include <sys/param.h>
    #ifdef __DragonFly__
    printf("DragonFly");
    #elif defined(__FreeBSD__)
    printf("FreeBSD");
    #elif defined(__NetBSD__)
    printf("NetBSD");
    #elif defined(__OpenBSD__)
    printf("OpenBSD");
    #else
    printf("BSD");
    #endif
    #endif
    // HP-UX (Hewlett-Packard Unix)
    #ifdef __hpux
    printf("HP-UX");
    #endif
    // Linux
    #ifdef __linux__
    printf("Linux");
    #endif
    // Solaris
    #if defined(__sun) && defined(__SVR4)
    printf("Solaris");
    #endif
    // Cygwin
    #if defined(__CYGWIN__) && !defined(_WIN32)
    printf("Cygwin");
    #endif
    // MingW
    #ifdef __MINGW64__
    printf("MingW on Microsoft Windows (64-bit)");
    #elif defined(__MINGW32__)
    printf("MingW on Microsoft Windows (32-bit)");
    #endif
    // Windows
    #ifdef _WIN64
    printf("Microsoft Windows (64-bit)");
    #elif defined(_WIN32)
    printf("Microsoft Windows (32-bit)");
    #endif

    printf("\n");
    return 0;
}
