/**
@author Devyn Collier Johnson <DevynCJohnson@Gmail.com>
@copyright LGPLv3

@brief statvfs() and fstatvfs() - return filesystem statistics
@version 2016.01.01

@section LICENSE
GNU Lesser General Public License v3
Copyright (c) Devyn Collier Johnson, All rights reserved.

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 3.0 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library.
*/

// gcc -Wall -Wextra -pedantic -O3 -Wl,-O3 -g0 -ggdb0 -s -fwhole-program -funroll-loops -pthread -pipe -ffunction-sections -fdata-sections -std=gnu11 -o ./statvfs ./statvfs.c && strip --remove-section=.note --remove-section=.comment --strip-debug --strip-unneeded ./statvfs


#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/statvfs.h>
#include <sys/types.h>


int main(int argc, char *argv[]) {
    int ret;
    struct statvfs buf;
    unsigned long long freespace, storagesize, usedspace;
    if (argc != 2) {
        fprintf(stderr, "Expected one parameter, but was given %d\n", argc - 1);
        return 1;
    } else if (0==(strncmp(argv[1], "/", 1))) {
        const char *path = argv[1];
        ret = statvfs(path,&buf);
    } else {
        int fd = atoi(argv[1]);
        ret = fstatvfs(fd,&buf);
    }
    if (ret > 0) {return ret;}
    storagesize = (unsigned long long)(buf.f_blocks * buf.f_frsize);
    freespace = (unsigned long long)(buf.f_bfree * buf.f_frsize);
    usedspace = storagesize - freespace;
    printf("Filesystem ID: %ld\n", buf.f_fsid);
    printf("Mount Flags: %ld\n", buf.f_flag);
    printf("Max Filename Length: %ld\n", buf.f_namemax);
    printf("Filesystem Size (based on fragments): %ld\n", buf.f_blocks);
    printf("Fragment Size (bytes): %ld\n", buf.f_frsize);
    printf("Block Size (bytes): %ld\n", buf.f_bsize);
    printf("Free Blocks: %ld\n", buf.f_bfree);
    printf("Free Blocks (non-root users): %ld\n", buf.f_bavail);
    printf("Total Inodes: %ld\n", buf.f_files);
    printf("Free Inodes: %ld\n", buf.f_ffree);
    printf("Free Inodes (non-root users): %ld\n", buf.f_favail);
    printf("Storage Size (bytes): %llu\n", storagesize);
    printf("Free Space (bytes): %llu\n", freespace);
    printf("Used Space (bytes): %llu\n", usedspace);
    return ret;
}
