/**
@author Devyn Collier Johnson <DevynCJohnson@Gmail.com>
@copyright LGPLv3

@brief Convert between various temperature units
@version 2016.01.01
@usage tempconv CONV_TO CONV_FROM TEMP

@section LICENSE
GNU Lesser General Public License v3
Copyright (c) Devyn Collier Johnson, All rights reserved.

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 3.0 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library.
*/

// gcc -Wall -Wextra -pedantic -ffast-math -O3 -Wl,-O3 -g0 -ggdb0 -s -fwhole-program -funroll-loops -pthread -pipe -ffunction-sections -fdata-sections -std=c11 -o ./tempconv ./tempconv.c && strip --remove-section=.note --remove-section=.comment --strip-debug --strip-unneeded ./tempconv
// gcc -march=haswell -Wall -Wextra -pedantic -ffast-math -O3 -Wl,-O3 -g0 -ggdb0 -s -fwhole-program -funroll-loops -pthread -pipe -ffunction-sections -fdata-sections -std=c11 -o ./tempconv ./tempconv.c && strip --remove-section=.note --remove-section=.comment --strip-debug --strip-unneeded ./tempconv


/* INITIALIZATION */


#include <stdio.h>
#include <stdlib.h>

#include "MACROS.h"


double c2k(double input);
double c2f(double input);
double f2c(double input);
double f2k(double input);
double k2c(double input);
double k2f(double input);


/* CONVERSION FUNCTIONS */


double c2k(double input) {
    return (input + 273.15);
}


double c2f(double input) {
    return ((input * 1.8) + 32);
}


double f2c(double input) {
    return ((input - 32.0) * 0.55556);
}


double f2k(double input) {
    return ((input + 459.67) * 0.55556);
}


double k2c(double input) {
    return (input - 273.15);
}


double k2f(double input) {
    return ((input - 273.15) * 1.8 + 32.0);
}


/* MAIN */


int main(int argc, char *argv[]) {
    if (argc != 4) {
        fprintf(stderr, "Expected 3 parameters (i.e. \"f c 100\"), but was given %d\n", argc - 1);
        return 1;
    }
    char conv_to = (char) argv[1][0];
    char conv_from = (char) argv[2][0];
    double num = strtod(argv[3], (char **)NULL);
    if ((conv_from == 'f' || conv_from == 'F') && (conv_to == 'c' || conv_to == 'C')) {
        printf("%.2f", f2c(num));
    } else if ((conv_from == 'f' || conv_from == 'F') && (conv_to == 'k' || conv_to == 'K')) {
        printf("%.2f", f2k(num));
    } else if ((conv_from == 'k' || conv_from == 'K') && (conv_to == 'c' || conv_to == 'C')) {
        printf("%.2f", k2c(num));
    } else if ((conv_from == 'k' || conv_from == 'K') && (conv_to == 'f' || conv_to == 'F')) {
        printf("%.2f", k2f(num));
    } else if ((conv_from == 'c' || conv_from == 'C') && (conv_to == 'k' || conv_to == 'K')) {
        printf("%.2f", c2k(num));
    } else if ((conv_from == 'c' || conv_from == 'C') && (conv_to == 'f' || conv_to == 'F')) {
        printf("%.2f", c2f(num));
    } else {
        fprintf(
            stderr,
            "Invalid parameters given!\nTo convert Celsius to Kelvin:\n'tempconv k c 100'\n"
        );
        return 1;
    }
    printf("\n");
    return 0;
}
